local addonName, ns = ...

-- Add-on nameSpace
local TAS = LibStub ("AceAddon-3.0"):NewAddon (addonName, "AceEvent-3.0")
local L = LibStub ("AceLocale-3.0"):GetLocale (addonName)
local Dialog 							-- Placeholder for LibStub("AceConfigDialog-3.0")


-- Constants
local TIER_MAX = MAX_TALENT_TIERS or 7
local COL_MAX = NUM_TALENT_COLUMNS or 3
local PVP_TIER_MAX = MAX_PVP_TALENT_TIERS or 6
local PVP_COL_MAX = MAX_PVP_TALENT_COLUMNS or 3
local SLASH_CASTUSE = {[1] = SLASH_CAST1, [2] = SLASH_CAST2, [3] = SLASH_CAST3, [4] = SLASH_CAST4, [5] = SLASH_USE1, [6] = SLASH_USE2}

-- Frames
local F_Temp = CreateFrame ("frame")	-- frame for onUpdate and Blizzard menu

-- Tables
TAS.talents = { }
-- Variables
local options 							-- Placeholder for the options table
local totalTime = 0 					-- onUpdate time holder

local defaults = {
	profile = {
		enabled = true,
		actionBarEnabled = true,
		macroEnabled = true,
		chatOutput = false,
	},
	class = {
		blacklistMacro = { },
		blacklistSpell = { },
	},
}

--**********************************************************************************************************************************************************
-------------------------------------------------------------- Initialisation Functions
--**********************************************************************************************************************************************************

function TAS:OnInitialize ()
	TAS.db = LibStub ("AceDB-3.0"):New ("TalentAutoSwitchDB", defaults, true)
	
	TAS.db.RegisterCallback (TAS, "OnProfileChanged", "OnProfileChanged")
	TAS.db.RegisterCallback (TAS, "OnProfileCopied", "OnProfileChanged")
	TAS.db.RegisterCallback (TAS, "OnProfileReset", "OnProfileChanged")
	
	-- OnDemand option load into memory
	F_Temp.name = addonName	
	local button = CreateFrame ("Button", nil, F_Temp, "OptionsButtonTemplate")
	button:SetPoint ("CENTER")
	button:SetText ("/tas")
	button:SetScript ("OnClick", function ()
			InterfaceOptionsFrameCancel_OnClick ()
			HideUIPanel (GameMenuFrame)
			TAS:SetupOptions ()
	end)
	
	InterfaceOptions_AddCategory (F_Temp)
		
	SLASH_TAS1 = "/tas"
	SLASH_TAS2 = "/talentautoswitch"
	SlashCmdList ["TAS"] = function ()
		TAS:SetupOptions ()		
	end
end

function TAS:OnEnable ()
	TAS:RegisterEvent ("PLAYER_TALENT_UPDATE")					-- These events allways fire together. Don't need all 3
	-- TAS:RegisterEvent ("PLAYER_SPECIALIZATION_CHANGED")		-- These events allways fire together. Don't need all 3
	-- TAS:RegisterEvent ("ACTIVE_TALENT_GROUP_CHANGED")		-- These events allways fire together. Don't need all 3
	-- TAS:RegisterEvent ("LEARNED_SPELL_IN_TAB")				-- Doesn't allways fire
	
	if UnitLevel("player") < GetMaxPlayerLevel() then 
		IconIntroTracker:UnregisterEvent("SPELL_PUSHED_TO_ACTIONBAR")
	end
	
	TAS:fix ()
	
	-- Localization notice
	local loadedL = _G.GetLocale()
	if loadedL == "ptBR" or loadedL == "deDE" or loadedL == "itIT" or loadedL == "koKR" or loadedL == "esMX" or loadedL == "ruRU" or loadedL == "esES" then
        print("|cff20ff20Talent Auto Switch|r: Help Translate at the Project Site!")
    end
end

function TAS:OnDisable ()
	TAS:UnregisterEvent ("PLAYER_TALENT_UPDATE")
	print ("|cff20ff20Talent Auto Switch|r: "..L["Disabled"])
end

function TAS:OnProfileChanged()
end

--**********************************************************************************************************************************************************
-------------------------------------------------------------- Body
--**********************************************************************************************************************************************************

function TAS:print (st)
	if TAS.db.profile.chatOutput then print (st) end
end

function TAS:BlackList (item, list) 
	if list [item] then
		print (string.format ("|cff20ff20Talent Auto Switch|r: "..L["Removed %s from blacklist"], item))
		list [item] = nil
	else
		print (string.format ("|cff20ff20Talent Auto Switch|r: "..L["Blacklisted %s"], item))
		list [item] = item
	end
end

function TAS:fix() 
	for tier = 1, TIER_MAX do
		TAS.talents = {}
		local selTalentID, selSpellName, selSpellID
		
		for col = 1, COL_MAX do
			local tempID, tempName, _, tempSelected, _, tempSpellID = GetTalentInfo (tier, col, 1)
			if tempSelected then
				selTalentID, selSpellName, selSpellID = tempID, tempName, tempSpellID
			else
				TAS.talents[tempName] = true
			end
		end
		if selSpellName then
			TAS:talentChanged (selTalentID, selSpellName, selSpellID)
		end
	end
	
	for tier = 1, PVP_TIER_MAX do
		TAS.talents = {}
		local selTalentID, selSpellName, selSpellID
		
		for col = 1, PVP_COL_MAX do
			local tempID, tempName, _, tempSelected, _, tempSpellID = GetPvpTalentInfo (tier, col, 1)
			if tempSelected then
				selTalentID, selSpellName, selSpellID = tempID, tempName, tempSpellID
			else
				TAS.talents[tempName] = true
			end
		end
		if selSpellName then
			TAS:talentChanged (selTalentID, selSpellName, selSpellID)
		end
	end
end

function TAS:talentChanged (selTalentID, selSpellName, selSpellID) 
	if IsPassiveSpell (selSpellID) then return end -- Exit because you can't put passive spells on bars
	
	for h = 1,120 do
		local actionType, actionID = GetActionInfo (h)
		if actionType == "macro" and TAS.db.profile.macroEnabled then
			local name, _, body = GetMacroInfo (actionID)
			if not TAS.db.class.blacklistMacro [name] and body then
				for i,v in pairs(TAS.talents) do
					for j,w in pairs (SLASH_CASTUSE) do
						if string.find (body, w.."(.*)"..i) then
							body = string.gsub (body, i, selSpellName)
							EditMacro (actionID, nil, nil, body)
							TAS:print (string.format ("|cff20ff20Talent Auto Switch|r: "..L["Replaced %s in macro %s with %s"], "|cff9d9d9d"..i.."|r", "|cff9d9d9d"..name.."|r", "|cff9d9d9d"..selSpellName.."|r"))
							break
						end
					end
				end
			end
		end
		if actionType == "spell" and TAS.db.profile.actionBarEnabled then
			local name = GetSpellInfo (actionID)
			if not TAS.db.class.blacklistSpell [name] then
				for i,v in pairs(TAS.talents) do
					if name == i then
						PickupAction (h)
						ClearCursor()
						if GetTalentInfoByID(selTalentID) then 
							PickupTalent (selTalentID) 
						else
							PickupPvpTalent (selTalentID)
						end
						PlaceAction (h)
						TAS:print (string.format ("|cff20ff20Talent Auto Switch|r: "..L["Replaced %s with %s on action bar spot %s"], "|cff9d9d9d"..name.."|r", "|cff9d9d9d"..selSpellName.."|r", "|cff9d9d9d"..h.."|r"))
					end
				end
			end
		end
	end
	ClearCursor ()
end

local function LF_onUpdate (self, elapsed)
	totalTime = totalTime + elapsed
    if totalTime >= .4 then
		F_Temp:SetScript ("OnUpdate", nil)
		TAS:fix ()
		TAS:RegisterEvent ("PLAYER_TALENT_UPDATE")
    end
end

function TAS:PLAYER_TALENT_UPDATE () -- LEARNED_SPELL_IN_TAB DOES NOT FIRE ON SOME LEARNED SPELLS. BLIZZ PROBLEM. FIX UNTIL RESOLVED
	totalTime = 0
	TAS:UnregisterEvent ("PLAYER_TALENT_UPDATE")
	F_Temp:SetScript ("OnUpdate", LF_onUpdate)
end


--**********************************************************************************************************************************************************
-------------------------------------------------------------------- Options Table
--**********************************************************************************************************************************************************

function TAS:SetupOptions ()
	local selectedBlackListMacro, selectedBlackListSpell
	if not options then
		options = {
			name = addonName,
			type = "group",
			name = "Talent Auto Switch: "..GetAddOnMetadata (addonName, "Version"),
			get = function (info) return TAS.db.profile [info [#info]] end,
			set = function (info, v) TAS.db.profile [info [#info]] = v end,
			args = {
				general = {
					type = "group",
					name = L["General Settings"],
					order = 0,
					childGroups = "tab",
					args = {
						headerGroup = {
							type = "group",
							name = " ",
							order = 0,
							inline = true,
							args = {
								enabled = {
									type = 'toggle',
									name = L["Enable"],
									desc = L["Enable"].." Talent Auto Switch", 
									order = 1,
									set = function (info, v) TAS.db.profile [info [#info]] = v
										if v then TAS:Enable () else TAS:Disable () end
									end,
								},
								fixTalents = {
									type = "execute",
									name = L["Fix"],
									desc = L["Attempt to fix talents in macros and action bars"],
									order = -1,
									func = function () 
										TAS:fix ()
									end,
								},
							},
						},
						chatOutput = {
							type = 'toggle', width = "full",
							name = L["Report Talent Switches"],
							desc = "",
							order = 1,
						},
						actionBarEnabled = {
							type = 'toggle', width = "full",
							name = L["Replace talent spells on action bars"],
							desc = "",
							order = 2,
						},
						macroEnabled = {
							type = 'toggle', width = "full",
							name = L["Replace talent spell names in macros"],
							desc = "",
							order = 3,
						},
						blacklistMacro = {
							type = "group",
							name = L["Blacklist Macros"],
							args = {
								typeBListMacro = {
									type = 'input', width = 'double',
									name = L["Enter Macro"],
									desc = "",
									order = 1,
									get = function (info) return "" end,
									set = function (info, v) TAS:BlackList (v, TAS.db.class.blacklistMacro) end,
								},
								selectBListMacro = {
									type = 'select', width = 'double',
									name = L["Delete Macro"],
									desc = "",
									order = 5,
									values = TAS.db.class.blacklistMacro,
									get = function (info) return selectedBlackListMacro end,
									set = function (info, v) selectedBlackListMacro = v end,
								},
								deleteBListMacro = {
									type = "execute", width = 'half',
									name = L["Delete"],
									desc = "",
									order = 10,
									func = function() 
										TAS:BlackList (selectedBlackListMacro, TAS.db.class.blacklistMacro)
										selectedBlackListMacro = nil
									end,
								},
							},
						},
						blacklistSpell = {
							type = "group",
							name = L["Blacklist Spell"],
							args = {
								enterBListSpell = {
									type = 'input', width = 'double',
									name = L["Enter Spell"],
									desc = "",
									order = 1,
									get = function (info) return "" end,
									set = function (info, v) TAS:BlackList (v, TAS.db.class.blacklistSpell) end,
								},
								selectBListSpell = {
									type = 'select', width = 'double',
									name = L["Delete Spell"],
									desc = "",
									order = 5,
									values = TAS.db.class.blacklistSpell,
									get = function (info) return selectedBlackListSpell end,
									set = function (info, v) selectedBlackListSpell = v end,
								},
								deleteBListSpell = {
									type = "execute", width = 'half',
									name = L["Delete"],
									desc = "",
									order = -1,
									func = function() 
										TAS:BlackList (selectedBlackListSpell, TAS.db.class.blacklistSpell)
										selectedBlackListSpell = nil
									end,
								},
							},
						},
					},
				},
			},
		}
		options.args.profile = LibStub ("AceDBOptions-3.0"):GetOptionsTable (TAS.db)
		
		Dialog = LibStub ("AceConfigDialog-3.0")
		LibStub ("AceConfigRegistry-3.0"):RegisterOptionsTable (addonName, options)
		Dialog:AddToBlizOptions (addonName, options.args.general.name, addonName, "general")
		Dialog:AddToBlizOptions (addonName, options.args.profile.name, addonName, "profile")
		print ("|cff20ff20Talent Auto Switch|r: "..L["Option table now loaded into memory"])
	end
	
	if not Dialog.OpenFrames[addonName] then 
		Dialog:SetDefaultSize (addonName, 700, 580)
		Dialog:Open (addonName) 
	else 
		Dialog:Close (addonName) 
	end	
end